local ButtonControl = FlexBar2.OO.Mixin
	{
		"Load", -- Called when the class gets initiated, suposed to initialize everything
		"Create", -- Called by load if the saved settings say the class is to be created; not called otherwise
		"Destroy",
		"Show",
		"Hide",
		"Lock",
		"Unlock",
		"LockIcon",
		"UnlockIcon",
		"LoadCurrentTexture", -- Find out what texture is suposed to currently be on the button and load it.
		"LoadSettings", -- Read the current settings and applies each one of them in the correct order
		"SetDefaults", -- Load the default settings for this button (works as a reset function too)
		"SetPosition",
		"SetScale",
		"SetText",
		"Controller", -- The name of the controller
	};


function ButtonControl:Activate(Button)
	if(Button) then
		self:LoadSettings(Button);
	end
end


function ButtonControl:Show(save)
	self.Frame.Button:Show();
	self.Frame:Show();
	if(save) then
		self.Settings.Show = true;
	end
end

function ButtonControl:Hide(save)
	self.Frame.Button:Hide();
	self.Frame:Hide();
	if(save) then
		self.Settings.Show = false;
	end
end


function ButtonControl:Lock(save)
	self.Frame:SetHeight(1);
	self.Frame:SetWidth(1);
	if(save) then
		self.Settings.Lock = true;
	end
end

function ButtonControl:Unlock(save)
	self.Frame:SetHeight(10);
	self.Frame:SetWidth(10);
	if(save) then
		self.Settings.Lock = false;
	end
end


function ButtonControl:LockIcon(save)
end

function ButtonControl:UnlockIcon(save)
end


function ButtonControl:LoadCurrentTexture(onlyreturn)
	--[[
		The function will first check if there is any 'forced' texture, and use that one if there is.
		Then the function will check if the Button is bound to an action id, if it is then it will use that texture.
		After that, if the button is bound to a spell, it takes that spell's texture.
		If onlyreturn is true then the texture will be returned but not set on the button
	]]
	if(self.Settings.Texture) then
		if(not onlyreturn) then self.Frame.Button.Icon:SetTexture(self.Setting.Texture); end
		return self.Settings.Texture;
	elseif(type(self.Settings.Attributes) == "table") then
		if(self.Settings.Attributes.type == "action" and self.Settings.Attributes.action) then
			if(not onlyreturn) then self.Frame.Button.Icon:SetTexture(GetActionTexture(self.Settings.Attributes.action)); end
			return GetActionTexture(self.Settings.Attributes.action);
		elseif(self.Settings.Attributes.type == "spell" and self.Settings.Attributes.spell) then
			local OK, ret = pcall(FlexBar2.BabbleSpell.GetSpellIcon, self.Settings.Attributes.spell); -- We use a pcall here in case a spell is not in BabbleSpell
			if(OK) then
				if(not onlyreturn) then self.Frame.Button.Icon:SetTexture(ret); end
				return ret;
			else
				FlexBar2:Debug("Failed to get the texture for the spell '" .. self.Settings.Attributes.spell .."' on button " .. self.Button);
				return false, ret;
			end
		end
	else
		return nil;
	end
end


function ButtonControl:LoadSettings(Button)
	Button = Button or self.Button;
	if(Button == nil) then return false; end
	self.Button = Button;
	if(type(FlexBar2.db.profile.Button[Button]) ~= "table") then
		FlexBar2.db.profile.Button[Button] = {};
	end
	self.Settings = FlexBar2.db.profile.Button[Button];
	if(self.Settings.Show == true) then
		self:Show();
	else
		self:Hide();
	end
	if(self.Settings.Lock == true) then
		self:Lock();
	else
		self:Unlock();
	end
	if(self.Settings.LockIcon == true) then
		self:LockIcon();
	else
		self:UnlockIcon();
	end
	if(type(self.Settings.Scale) == "number") then
		self:SetScale(self.Settings.Scale);
	end
	if(type(self.Settings.X) == "number" and type(self.Settings.Y) == "number") then
		self:SetPosition(self.Settings.X, self.Settings.Y);
	end
	if(type(self.Settings.Attributes) == "table") then
		for n, v in pairs(self.Settings.Attributes) do
			self.Frame.Button:SetAttribute(n, v);
		end
	end
	self:LoadCurrentTexture();
	return true;
end


function ButtonControl:SetDefaults()
	local X, Y, Attributes, Numb;
	if(self.Button == nil) then return false; end
	if(type(self.Button) == "number") then
		Numb = self.Button;
	else
		Numb = 1;
	end
	X = 100 + (38 * (Numb - ((math.ceil(Numb / 12) - 1) * 12)));
	Y = 100 + (38 * (math.ceil(Numb / 12))); -- So anoying that Y has to be negative, but nothing I can do about it as long as I anchor to TOPLEFT
	if(Numb >= 1 and Numb <= 120) then -- Only set the attributes for a button to act like an actionbutton if the button's number would be a valid action id
		Attributes =
		{
			action = Numb,
			type = "action",
		}
	end
	FlexBar2.db.profile.Button[self.Button] =  -- We set new references here, so we must watch out
	{
		Show = true,
		Lock = false,
		LockIcon = false,
		Scale = 1,
		X = X,
		Y = Y,
		Attributes = Attributes,
	};
	self.Settings = FlexBar2.db.profile.Button[self.Button];
	return true;
end

function ButtonControl:SetPosition(X, Y, save)
	self.Frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", X, -Y);
	if(save) then
		self.Settings.X = X;
		self.Settings.Y = Y;
	end
end


function ButtonControl:SetScale(Scale, save)
	self.Frame:SetScale(Scale);
	if(save) then
		self.Settings.Scale = Scale;
	end
end


function ButtonControl:SetText()
end


FlexBar2:RegisterClass(ButtonControl);
